package gov.hud.arm.service;

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.List;

import javax.xml.ws.soap.MTOM;

import org.apache.commons.codec.Charsets;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

@MTOM
public class ArmServiceImpl implements ArmService{

	public String ping(final Ping ping) {
		System.out.println("Invoking 'ping'");
		System.out.println("Invoked 'ping'");
		return "ARM Service Version 15.2.6.1 (Latest Databag Version 6.0) - " + ping.getAgcHcsId();
	}

	public GetReferenceResponse getReference(final GetReference getReference) {
		System.out.println("Invoking 'getReference'");
		System.out.println("Request is: " + ToStringBuilder.reflectionToString(getReference, ToStringStyle.MULTI_LINE_STYLE));
		final GetReferenceResponse response = new GetReferenceResponse();
		final ReferenceItem value = new ReferenceItem();
		value.setId(1);
		value.setLongDesc("ARM SDK Test Reference Item");
		value.setName("Test Reference Item 1");
		value.setShortDesc("Reference Item");
		final List<ReferenceItem> referenceItems = response.getReferenceItem();
		referenceItems.add(value);
		value.setId(2);
		value.setName("Test Reference Item 2");
		referenceItems.add(value);
		System.out.println("Invoked 'getReference'");
		return response;
	}

	public GetAllReferenceItemsResponse getAllReferenceItems(final GetAllReferenceItems getReference) {
		System.out.println("Invoking 'getAllReferenceItems'");
		System.out.println("Request is: " + ToStringBuilder.reflectionToString(getReference, ToStringStyle.MULTI_LINE_STYLE));
		final GetAllReferenceItemsResponse response = new GetAllReferenceItemsResponse();
		final List<ReferenceItemGroup> referenceItemGroups = response.getReferenceItemGroup();

		final ReferenceItemGroup value = new ReferenceItemGroup();
		value.setGroupId(30);
		
		final List<ReferenceItem> refItems = value.getReferenceItem();
		ReferenceItem refItem = new ReferenceItem();
		refItem.setId(1);
		refItem.setLongDesc("ARM SDK Test Reference Item");
		refItem.setName("Test Reference Item 1");
		refItem.setShortDesc("Reference Item");
		refItems.add(refItem);

		refItem = new ReferenceItem();
		refItem.setId(2);
		refItem.setLongDesc("ARM SDK Test Reference Item");
		refItem.setName("Test Reference Item 2");
		refItem.setShortDesc("Reference Item");
		refItems.add(refItem);

		referenceItemGroups.add(value);
		
		System.out.println("Invoked 'getAllReferenceItems'");
		return response;
	}

	public GetReferenceGroupNamesResponse getReferenceGroupNames(final GetReferenceGroupNames request) {
		System.out.println("Invoking 'getReferenceGroupNames'");
		final GetReferenceGroupNamesResponse response = new GetReferenceGroupNamesResponse();
		final List<ReferenceGroup> groups = response.getReferenceGroup();
		final ReferenceGroup group = new ReferenceGroup();
		group.setId(0);
		group.setName("States");
		groups.add(group);
		return response;
	}	

	public PostSubmissionResponse postSubmission(final PostSubmission postSubmission) {
		System.out.println("Invoking 'postSubmission'");
		try {
			final InputStream is = postSubmission.getSubmissionData().getDataSource().getInputStream();
			try {
				final String data = IOUtils.toString(is, Charsets.UTF_8);
				System.out.println("Data on request is: " + data);
			} finally {
				is.close();
			}
		} catch (final IOException e) {
			e.printStackTrace();
		}
		final PostSubmissionResponse response = new PostSubmissionResponse();
		response.setSubmissionId(12345678);
		System.out.println("Invoked 'postSubmission'");
		return response;
	}

	public GetSubmissionInfoResponse getSubmissionInfo(final GetSubmissionInfo getSubmissionInfo) {
		System.out.println("Invoking 'getSubmissionInfo'");
		System.out.println("Request is: " + ToStringBuilder.reflectionToString(getSubmissionInfo, ToStringStyle.MULTI_LINE_STYLE));
		final GetSubmissionInfoResponse response = new GetSubmissionInfoResponse();
		response.setStatusDate(new Date());
		response.setStatusMessage("A submission status message");
		System.out.println("Invoked 'getSubmissionInfo'");
		return response;
	}

	public GetDetailedErrorMessageResponse getDetailedErrorMessage(
			final GetDetailedErrorMessage msg) {
		System.out.println("Invoking 'getDetailedErrorMessage'");
		System.out.println("Request is: " + ToStringBuilder.reflectionToString(msg, ToStringStyle.MULTI_LINE_STYLE));
		final GetDetailedErrorMessageResponse response = new GetDetailedErrorMessageResponse();
		response.setStatusDate(new Date());
		response.setMessage("An error message");
		System.out.println("Invoked 'getDetailedErrorMessage'");
		return response;
	}

	public void refreshReferenceItemCache(final RefreshReferenceItemCache msg) {
		System.out.println("Invoking 'refreshReferenceItemCache' for reference Item = " + msg.getReferenceId());
	}

	public void refreshAllReferenceItemsInCache(final RefreshAllReferenceItemsInCache msg) {
		System.out.println("Invoking 'refreshAllReferenceItemsInCache'");
	}

	public PostSubmissionResponse postAgencyData(final PostAgencyData agencyData) {
		System.out.println("Invoking 'postAgencyData'");
		try {
			final InputStream is = agencyData.getSubmissionData().getDataSource().getInputStream();
			try {
				final String data = IOUtils.toString(is, Charsets.UTF_8);
				System.out.println("Data on request is: " + data);
			} finally {
				is.close();
			}
		} catch (final IOException e) {
			e.printStackTrace();
		}
		final PostSubmissionResponse response = new PostSubmissionResponse();
		response.setSubmissionId(12345678);
		System.out.println("Invoked 'postAgencyData'");
		return response;
	}

	public PostSubmissionResponse postClientData(final PostClientData postClientData) {
		System.out.println("Invoking 'postClientData'");
		try {
			final InputStream is = postClientData.getSubmissionData().getDataSource().getInputStream();
			try {
				final String data = IOUtils.toString(is, Charsets.UTF_8);
				System.out.println("Data on request is: " + data);
			} finally {
				is.close();
			}
		} catch (final IOException e) {
			e.printStackTrace();
		}
		final PostSubmissionResponse response = new PostSubmissionResponse();
		response.setSubmissionId(12345678);
		System.out.println("Invoked 'postClientData'");
		return response;
	}

	public PostSubmissionResponse postCounselorData(
			final PostCounselorData postCounselorData) {
		System.out.println("Invoking 'postCounselorData'");
		try {
			final InputStream is = postCounselorData.getSubmissionData().getDataSource().getInputStream();
			try {
				final String data = IOUtils.toString(is, Charsets.UTF_8);
				System.out.println("Data on request is: " + data);
			} finally {
				is.close();
			}
		} catch (final IOException e) {
			e.printStackTrace();
		}
		final PostSubmissionResponse response = new PostSubmissionResponse();
		response.setSubmissionId(12345678);
		System.out.println("Invoked 'postCounselorData'");
		return response;
	}

	public PostSubmissionResponse postForm9902Data(
			final PostForm9902Data postForm9902Data) {
		System.out.println("Invoking 'postForm9902Data'");
		try {
			final InputStream is = postForm9902Data.getSubmissionData().getDataSource().getInputStream();
			try {
				final String data = IOUtils.toString(is, Charsets.UTF_8);
				System.out.println("Data on request is: " + data);
			} finally {
				is.close();
			}
		} catch (final IOException e) {
			e.printStackTrace();
		}
		final PostSubmissionResponse response = new PostSubmissionResponse();
		response.setSubmissionId(12345678);
		System.out.println("Invoked 'postForm9902Data'");
		return response;
	}
}
